__title__ = 'DRF Json Api Utils'
__version__ = '1.2.1'
__author__ = 'Amit Assaraf'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2020 Amit Assaraf'

# Version synonym
VERSION = __version__

# Header encoding (see RFC5987)
HTTP_HEADER_ENCODING = 'iso-8859-1'

# Default datetime input and output formats
ISO_8601 = 'iso-8601'

from .factory import *
